------------------------------------------------------------
-- Core.lua
--
-- Abin
-- 2010-10-24
------------------------------------------------------------

local type = type
local hooksecurefunc = hooksecurefunc
local ipairs = ipairs
local sort = sort
local GetAddOnMetadata = GetAddOnMetadata
local _G = _G

local L = CompactRaid:GetLocale("Artwork")
if type(CompactRaid.CheckVersion) ~= "function" or not CompactRaid:CheckVersion("1.3", L["title"]) then return end
local module = CompactRaid:RegisterModule("Artwork", L["title"], "1.3", L["desc"])
if not module then return end

local manager = CompactRaid.buttonManager
local layout = CompactRaid.layoutManager

local BACKDROP = { bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16, insets = {left = 5, right = 5, top = 5, bottom = 5 }}

local DEFAULT_ELEMENTS = {
	statusbar = "Interface\\RaidFrame\\Raid-Bar-Hp-Fill",
	font = STANDARD_TEXT_FONT,
	border = "Interface\\Tooltips\\UI-Tooltip-Border",
	background = "Interface\\DialogFrame\\UI-DialogBox-Background",
}

local function SetBlizzardElement(overlay, category, file)
	local frame = overlay:GetParent()
	if category == "statusbar" then
		manager.SetBothBarTexture(frame, file)
	elseif category == "font" then
		local height = CompactRaid.db.nameHeight or 12
		frame.name:SetFont(file, height)
		frame.statusText:SetFont(file, height * 0.84)
	end
end

local function ApplyElement(category, file)
	module.db[category] = file
	if category == "statusbar" or category == "font" then
		SetBlizzardElement(CompactRaid.sampleOverlay, category, file)
		CompactRaid:EnumFrames(SetBlizzardElement, category, file)
	elseif category == "border" or category == "background" then
		BACKDROP[category == "border" and "edgeFile" or "bgFile"] = file
		layout:UpdateContainers("SetBackdrop", BACKDROP)
		layout:UpdateContainers("UpdateAlpha")
	end
end

local function AddElement(category, id, key, file)
	id = id + 1
	if not module.listSelection and file == module.db[category] then
		module.listSelection = id
	end
	module.list:InsertData({ category = category, id = id, key = key, file = file })
	return id
end

function module:LoadCategory(category)
	self.listSelection = nil
	self.list:Clear()

	local lib = self.library
	local id = AddElement(category, 0, DEFAULT, DEFAULT_ELEMENTS[category])

	if lib and type(lib.List) == "function" and type(lib.Fetch) == "function" then
		local mt = lib:List(category)
		local key, file
		for _, key in ipairs(mt) do
			local file = lib:Fetch(category, key)
			id = AddElement(category, id, key, file)
		end
	elseif lib and type(lib.registry) == "table" then
		local temp = {}
		local data
		for _, data in ipairs(lib.registry) do
			if data[1] == category then
				local key, file = data[2], data[3]
				if type(key) == "string" and type(file) == "string" then
					tinsert(temp, { key = key, file = file })
				end
			end
		end

		sort(temp, function(t1, t2) return t1.key < t2.key end) -- Dude, I have to sort for ya!

		for _, data in ipairs(temp) do
			id = AddElement(category, id, data.key, data.file)
		end
	end

	local position = self.listSelection or 1
	self.list:SetSelection(position)
	self.list:RefreshContents()
	self.list:EnsureVisible()
end

local function SearchSharedMediaLib()
	local name, lib, ver
	if LibStub then
		name = "LibSharedMedia-3.0"
		lib, ver = LibStub(name, 1)
		if not lib then
			name = "LibSharedMedia-2.0"
			lib, ver = LibStub(name, 1)
		end
	end

	if lib then
		if ver then
			ver = "r"..ver
		end
	elseif CWDGMediaPack then
		name, lib, ver = "CWDGMediaPack", CWDGMediaPack
	end

	if lib then
		return lib, name, (GetAddOnMetadata(name, "Version") or "")..(ver and " "..ver or "")
	end
end

local function ValidateFile(category)
	local valid
	local file = module.db[category]
	if type(file) == "string" then
		if category == "font" then
			valid = module.testFont:SetFont(file, 10)
		else
			valid = module.testTexture:SetTexture(file)
		end
	end

	if not valid then
		module.db[category] = DEFAULT_ELEMENTS[category]
	end

	return valid
end

function module:OnInitialize(db)
	-- Validate saved file from last game session
	ValidateFile("statusbar")
	ValidateFile("font")
	ValidateFile("border")
	ValidateFile("background")

	-- search for loaded media libraries
	local lib, name, ver = SearchSharedMediaLib()
	if lib then
		self.library = lib
		self.libName:SetFormattedText("%s |cff808080%s|r", name, ver)
	else
		self.libName:SetText(NONE)
		self.libName:SetTextColor(0.5, 0.5, 0.5)
	end

	self.tabFrame:SelectTab(1) -- statusbar applied automatically

	ApplyElement("statusbar", db.statusbar)
	ApplyElement("font", db.font)
	ApplyElement("border", db.border)
	ApplyElement("background", db.background)

	hooksecurefunc(manager, "OnStatusBarTextureSet", function(self)
		manager.SetBothBarTexture(self, db.statusbar)
	end)

	hooksecurefunc(manager, "OnNameTextHeight", function(self)
		local height = CompactRaid.db.nameHeight or 12
		self.name:SetFont(db.font, height)
		self.statusText:SetFont(db.font, height * 0.84)
	end)
end

local page = module.optionPage
page.sampleX, page.sampleY = 325, -108

function page:OnSampleShow() end

module.testTexture = page:CreateTexture()
module.testFont = page:CreateFontString()

local libTitle = page:CreateFontString(nil, "ARTWORK", "GameFontNormalLeft")
page:AnchorToTopLeft(libTitle, 0, -8)
libTitle:SetText(L["library"])

local libName = page:CreateFontString(nil, "ARTWORK", "GameFontHighlightLeft")
module.libName = libName
libName:SetPoint("LEFT", libTitle, "LEFT", 84, 0)

local selectionTitle = page:CreateFontString(nil, "ARTWORK", "GameFontNormalLeft")
selectionTitle:SetPoint("TOPLEFT", libTitle, "BOTTOMLEFT", 0, -8)
selectionTitle:SetText(L["selection"])

local selectionText = page:CreateFontString(nil, "ARTWORK", "GameFontHighlightLeft")
selectionText:SetPoint("LEFT", selectionTitle, "LEFT", 84, 0)

local tabFrame = UICreateTabFrame(page:GetName().."TabFrame", page)
module.tabFrame = tabFrame
tabFrame:SetSize(380, 256)
tabFrame:SetPoint("TOPLEFT", selectionTitle, "BOTTOMLEFT", 0, -30)
page:CreatePanel(tabFrame)
tabFrame:SetBackdropBorderColor(0, 0, 0, 0)

tabFrame:AddTab(L["statusbar"], "statusbar")
tabFrame:AddTab(L["font"], "font")
tabFrame:AddTab(L["border"], "border")
tabFrame:AddTab(L["background"], "background")

function tabFrame:OnTabSelected(id, category)
	module:LoadCategory(category)
end

local list = UICreateVirtualScrollList(tabFrame:GetName().."List", tabFrame, 12, 1)
module.list = list
list:SetPoint("TOPLEFT", 5, -7)
list:SetPoint("BOTTOMRIGHT", -7, 7)

function list:OnButtonCreated(button)
	local frame = CreateFrame("Frame", nil, button)
	button.frame = frame
	frame:SetSize(180, 45)
	frame:SetScale(0.4)
	frame:SetPoint("LEFT", 16, 0)

	local icon = button:CreateTexture(nil, "ARTWORK")
	button.icon = icon
	icon:SetAllPoints(frame)
	icon:SetVertexColor(0, 1, 0)

	local name = button:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
	button.name = name
	name:SetText(NAME.."Abc123")
	name:SetPoint("LEFT", frame, "LEFT")

	local idText = button:CreateFontString(nil, "ARTWORK", "GameFontHighlightRight")
	button.idText = idText
	idText:SetWidth(24)
	idText:SetPoint("LEFT", frame, "RIGHT", 12, 0)

	local text = button:CreateFontString(nil, "ARTWORK", "GameFontHighlightLeft")
	button.text = text
	text:SetPoint("LEFT", idText, "RIGHT", 12, 0)
end

local SAMPLE_BACKDROP = {}

function list:OnButtonUpdate(button, data)
	local frame = button.frame
	local icon = button.icon
	local name = button.name
	frame:Hide()
	icon:Hide()
	name:Hide()

	local category = data.category
	if category == "statusbar" then
		icon:SetTexture(data.file)
		icon:Show()
	elseif category == "font" then
		name:SetFont(data.file, 12)
		name:Show()
	elseif category == "border" then
		SAMPLE_BACKDROP.bgFile = ""
		SAMPLE_BACKDROP.edgeFile = data.file
		frame:SetBackdrop(SAMPLE_BACKDROP)
		frame:Show()
	elseif category == "background" then
		SAMPLE_BACKDROP.bgFile = data.file
		SAMPLE_BACKDROP.edgeFile = ""
		frame:SetBackdrop(SAMPLE_BACKDROP)
		frame:Show()
	end

	--button.icon:SetTexture(data.file)
	button.text:SetText(data.key)
	button.idText:SetFormattedText("%d", data.id)
end

function list:OnSelectionChanged(position, data)
	if data then
		selectionText:SetFormattedText("%d  %s", position, data.key)
		if module.db[data.category] ~= data.file then
			module.db[data.category] = data.file
			ApplyElement(data.category, data.file)
		end
	end
end

function page:OnInitShow()
	list:EnsureVisible()
end