﻿------------------------------------------------------------
-- Core.lua
--
-- Abin
-- 2010/10/16
------------------------------------------------------------

local floor = floor
local GetTime = GetTime
local CreateFrame = CreateFrame
local strsub = strsub
local strfind = strfind
local ipairs = ipairs
local pairs = pairs
local type = type
local tostring = tostring
local UnitBuff = UnitBuff
local UnitDebuff = UnitDebuff
local UnitPowerType = UnitPowerType
local select = select
local UnitClass = UnitClass
local tinsert = tinsert
local format = format

local L = CompactRaid:GetLocale("CornerIndicators")
local module = CompactRaid:RegisterModule("CornerIndicators", L["title"], "1.3", L["desc"], 1)
if not module then return end

module.INDICATOR_KEYS = { "TOPLEFT", "TOPRIGHT", "BOTTOMLEFT", "BOTTOMRIGHT" }
module.indicators = {}

local function FindAura(unit, aura, selfcast, lacks, similar)
	local filter = selfcast and "PLAYER" or nil
	local harmful
	local name, _, icon, count, _, duration, expires = UnitBuff(unit, aura, nil, filter)
	if not name then
		harmful = 1
		name, _, icon, count, _, duration, expires = UnitDebuff(unit, aura, nil, filter)
	end

	-- Aura not found, try checking for similar auras from others
	if not name and similar then
		name, icon, count, duration, expires, harmful = module:FindSimilarBuff(unit, aura)
	end

	if lacks then
		if not name then
			return "", nil, 1, 0, 0, 1 -- Lacks treated as harmful, displays in red always
		end
	else
		if name then
			return name, icon, count or 1, duration or 0, expires or 0, harmful
		end
	end
end

local function Indicator_GetData(self, stages)
	local data = module.talentdb and module.talentdb[self.key]
	if stages then
		local stage1 = data and data["1"]
		local stage2 = data and data["2"]
		return data, stage1, stage2
	else
		return data
	end
end

local function Indicator_UpdateStatus(self)
	local count, duration, expires = self.count, self.duration, self.expires
	if not count or not duration or not expires then
		return
	end

	local timeLeft
	if duration > 0 and expires > 0 then
		timeLeft = expires - GetTime()
		if timeLeft < 0 then
			timeLeft = 0
		end
	end

	local data, stage1, stage2 = Indicator_GetData(self, 1)
	if not data or not stage1 or not stage2 then
		return
	end

	if self.isSample then
		if not timeLeft or timeLeft == 0 then
			self:UpdateAura()
			return
		end

		if data.style == 0 then
			return
		end
	end

	timeLeft = timeLeft and	floor(timeLeft + 0.501) -- 0.501 compensates LUA floating representation of 1 as 0.99999999725, does 0.5 work as well? I'm not sure, better careful than sorry

	local r, g, b
	if self.harmful then
		r, g, b = 1, 0, 0 -- debuff or lacks color, red always
	else
		r, g, b = data.r, data.g, data.b
		if timeLeft then
			-- Temproray aura
			if stage2.threshold and timeLeft <= stage2.threshold then
				r, g, b = stage2.r, stage2.g, stage2.b
			elseif stage1.threshold and timeLeft / duration * 100 <= stage1.threshold then
				r, g, b = stage1.r, stage1.g, stage1.b
			end
		end
	end

	local text = self.text
	self.icon:SetVertexColor(r, g, b)
	text:SetVertexColor(r, g, b)

	if self.prevtimeLeft ~= timeLeft or self.prevCount ~= count then
		self.prevtimeLeft, self.prevCount = timeLeft, count
		if timeLeft then
			if count > 1 then
				text:SetFormattedText("%d-%d", count, timeLeft)
			else
				text:SetFormattedText("%d", timeLeft)
			end
		else
			text:SetText("N")
		end
	end
end

local function Indicator_OnUpdate(self, elapsed)
	self.elapsed = (self.elapsed or 0) + elapsed
	if self.elapsed < 0.2 then
		return
	end
	self.elapsed = 0
	Indicator_UpdateStatus(self)
end

local function Indicator_UpdateAura(self)
	self:Hide()
	if self.disabled then
		return
	end

	local data = Indicator_GetData(self)
	if not data or not data.aura then
		return
	end

	local parent = self:GetParent()
	local unit = parent.unit
	if not unit or not UnitIsConnected(unit) or UnitIsDead(unit) or UnitIsGhost(unit) then
		return
	end

	if (data.ignorePhysical and parent.category == 1) or (data.ignoreMagical and parent.category == 2) or (data.ignoreVehicle and parent.inVehicle) or (data.ignoreOutRanged and not parent.inRange) then
		return
	end

	local name, icon, count, duration, expires, harmful = FindAura(unit, data.aura, data.selfcast, data.showlacks, data.checkSimilar)
	if not name then
		return
	end

	self.count, self.duration, self.expires, self.harmful = count, duration, expires, harmful
	self.cooldown:Hide()
	self.countText:Hide()
	if data.style == 0 then
		self:SetScript("OnUpdate", nil)
		self.icon:SetTexture(icon or "Interface\\Icons\\INV_Misc_QuestionMark")
		self.icon:SetVertexColor(1, 1, 1)

		if count > 1 then
			self.countText:SetText(count)
			self.countText:Show()
		end

		if duration > 0 and expires > 0 then
			self.cooldown:SetCooldown(expires - duration, duration)
			self.cooldown:Show()
		end
	else
		self.icon:SetTexture(1, 1, 1)
		self:SetScript("OnUpdate", Indicator_OnUpdate)
		Indicator_UpdateStatus(self)
	end
	self:Show()
end

local function Indicator_SetStyle(self)
	local data = Indicator_GetData(self)
	if not data then
		return
	end

	local style = data.style
	if style == 3 then
		-- Hidden
		self.disabled = 1
		self:Hide()
	else
		if style == 2 then
			-- Numeric
			self.bkgnd:Hide()
			self.icon:Hide()
			self.text:Show()
		else
			if style == 1 then
				-- Cube
				self:SetSize(8, 8)
			else
				-- icon
				self:SetSize(12, 12)
			end

			self.bkgnd:Show()
			self.icon:Show()
			self.text:Hide()
		end

		self.disabled = nil
		self:Show()
		self:UpdateAura()
	end
end

local function Indicator_UpdateScale(self)
	local scale = module.chardb[self.key].scale
	if type(scale) ~= "number" or scale < 10 or scale > 300 then
		scale = 100
	end
	self:SetScale(scale / 100)
end

local function Indicator_UpdateOffset(self)
	local data = module.chardb[self.key]
	local xoffset, yoffset = data.xoffset, data.yoffset
	if type(xoffset) ~= "number" or  type(yoffset) ~= "number" then
		xoffset, yoffset = 0, 0
	end

	self:ClearAllPoints()
	self:SetPoint(self.key, xoffset + (self.leftSpacing or 0), yoffset)
end

-- Update the topleft indicator position when role/raid icon shows/hides, need to shift our indicator 13 pixels towards
-- right if either icon is shown, or shift back to topleft when both icons are hidden. Kinda aweful...
local function UpdateTopLeftPosition(overlay, hasRaidIcon, hasRoleIcon)
	local indicator = overlay.cornerIndicatorsFrame
	indicator = indicator and indicator.cornerIndicators.TOPLEFT
	if indicator then
		indicator.leftSpacing = (hasRaidIcon or hasRoleIcon) and 12 or nil
		indicator:UpdateOffset()
	end
end

function module:CreateIndicator(parent, key)
	local frame = CreateFrame("Frame", nil, parent)
	frame:SetSize(8, 8)
	frame:SetPoint(key)
	frame.key = key

	frame.bkgnd = frame:CreateTexture(nil, "BACKGROUND")
	frame.bkgnd:SetAllPoints(frame)
	frame.bkgnd:SetTexture(0, 0, 0, 1)

	frame.icon = frame:CreateTexture(nil, "BORDER")
	frame.icon:SetPoint("TOPLEFT", frame.bkgnd, "TOPLEFT", 1, -1)
	frame.icon:SetPoint("BOTTOMRIGHT", frame.bkgnd, "BOTTOMRIGHT", -1, 1)
	frame.icon:SetTexCoord(0.08, 0.92, 0.08, 0.92)
	frame.icon:SetTexture(1, 1, 1)

	frame.countText = frame:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
	frame.countText:SetFont(STANDARD_TEXT_FONT, 9, "OUTLINE")
	frame.countText:SetJustifyH("RIGHT")
	frame.countText:SetJustifyV("BOTTOM")
	frame.countText:SetPoint("CENTER", 2, -1)

	frame.text = frame:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
	frame.text:SetFont(STANDARD_TEXT_FONT, 9)
	frame.text:SetPoint(key, frame.icon, key)

	frame.cooldown = CreateFrame("Cooldown", nil, frame)
	frame.cooldown:SetAllPoints(frame.icon)
	frame.cooldown:Raise()
	frame.cooldown:SetReverse(true)
	frame.cooldown:SetDrawEdge(true)

	if strfind(key, "^TOP") then
		frame.text:SetJustifyV("TOP")
	elseif strfind(key, "^BOTTOM") then
		frame.text:SetJustifyV("BOTTOM")
	end

	if strfind(key, "LEFT$") then
		frame.text:SetJustifyH("LEFT")
	elseif strfind(key, "RIGHT$") then
		frame.text:SetJustifyH("RIGHT")
	end

	frame.SetStyle = Indicator_SetStyle
	frame.UpdateScale = Indicator_UpdateScale
	frame.UpdateOffset = Indicator_UpdateOffset
	frame.UpdateAura = Indicator_UpdateAura

	tinsert(self.indicators, frame)
	return frame
end

local SAMPLE_ICONS = {
	TOPLEFT = "Interface\\Icons\\Ability_Hunter_MarkedForDeath",
	TOPRIGHT = "Interface\\Icons\\Ability_Paladin_ProtectorOfTheInnocent",
	BOTTOMLEFT = "Interface\\Icons\\ABILITY_SEAL",
	BOTTOMRIGHT = "Interface\\Icons\\Spell_Magic_GreaterBlessingofKings",
}

function module:UpdateIndicator(indicator, style, aura, scale, offset)
	if style then
		indicator:SetStyle()
		aura = nil
	end

	if aura then
		indicator:UpdateAura()
	end

	if scale then
		indicator:UpdateScale()
	end

	if offset then
		indicator:UpdateOffset()
	end
end

function module:UpdateAllIndicators(key, style, aura, scale, offset)
	local indicator
	for _, indicator in ipairs(self.indicators) do
		if not key or indicator.key == key then
			self:UpdateIndicator(indicator, style, aura, scale, offset)
		end
	end
end

local function ValidateDB(db)
	local key
	for _, key in ipairs(module.INDICATOR_KEYS) do
		if type(db[key]) ~= "table" then
			db[key] = { r = 0, g = 1, b = 0 }
			local style = db[key].style
			if style ~= 0 and style ~= 1 and style ~= 2 and style ~= 3 then
				db[key].style = 1
			end

			if type(db[key]["1"]) ~= "table" then
				db[key]["1"] = { threshold = 50, r = 1, g = 1, b = 0 }
			end

			if type(db[key]["2"]) ~= "table" then
				db[key]["2"] = { threshold = 5, r = 1, g = 0, b = 0 }
			end

			db[key].scale, db[key].xoffset, db[key].yoffset = nil -- cleanup garbage from previous versions
		end
	end
end

function module:OnInitialize(db, chardb, firstTime)
	-- Create indicators for addon sample button
	local overlay = CompactRaid.sampleOverlay
	local frame = CreateFrame("Frame", nil, overlay)
	overlay.cornerIndicatorsFrame = frame
	self.sampleFrame = frame
	frame:Hide()
	frame:SetAllPoints(overlay)
	frame.cornerIndicators = {}

	local key
	for _, key in ipairs(self.INDICATOR_KEYS) do
		if type(chardb[key]) ~= "table" then
			chardb[key] = {}
		end

		local indicator = self:CreateIndicator(frame, key)
		frame.cornerIndicators[key] = indicator
		indicator.isSample = 1
		indicator.count = 1

		indicator.UpdateAura = function(self)
			local data = Indicator_GetData(self)
			if not data then
				return
			end

			local now = GetTime()
			local duration = random(10, 30)
			local expires = now + duration
			self.duration, self.expires = duration, expires

			if data.style == 0 then
				self.icon:SetTexture(SAMPLE_ICONS[self.key])
				self.icon:SetVertexColor(1, 1, 1)
				self.cooldown:SetCooldown(now, duration)
				self.cooldown:Show()
			else
				self.icon:SetTexture(1, 1, 1)
				self.icon:SetVertexColor(0, 1, 0)
				self.cooldown:Hide()
			end
		end

		indicator:SetScript("OnUpdate", Indicator_OnUpdate)
		indicator:Show()
	end

	hooksecurefunc(CompactRaid.sampleButton, "UpdateRaidIcon", function(self)
		UpdateTopLeftPosition(CompactRaid.sampleOverlay, self.roleIcon:IsShown())
	end)

	UpdateTopLeftPosition(CompactRaid.sampleOverlay, CompactRaid.sampleButton.roleIcon:IsShown())
end

function module:OnTalentGroupChange(talentGroup, talentdb, firstTime)
	if firstTime then
		CompactRaid:CopyTable(talentdb, self.defaultsDB)
	end

	ValidateDB(talentdb)

	local tabFrame = self.optionPage.tabFrame
	local selection = tabFrame:GetSelection()
	if selection == 0 then
		tabFrame:SelectTab(1)
	else
		tabFrame:OnTabSelected(selection)
	end
	self:UpdateAllIndicators(nil, 1, 1)
end

local function Frame_OnEvent(self, event, unit)
	if unit and self.unit and UnitIsUnit(unit, self.unit) then--and (UnitIsPlayer(unit) or UnitPlayerControlled(unit)) then
		if event == "UNIT_DISPLAYPOWER" and self.class == "DRUID" then
			self.category = (UnitPowerType(unit) == 0) and 2 or 1
		end

		local indicator
		for _, indicator in pairs(self.cornerIndicators) do
			indicator:UpdateAura()
		end
	end
end

local function Frame_UpdateAura(self)
	Frame_OnEvent(self, self.class == "DRUID" and "UNIT_DISPLAYPOWER" or "UNIT_AURA", self.unit)
end

local function Frame_OnShow(self)
	self:SetScript("OnEvent", Frame_OnEvent)
end

local function Frame_OnHide(self)
	self:SetScript("OnEvent", nil)
end

function module:OnCreate(overlay)
	local frame = CreateFrame("Frame", nil, overlay)
	overlay.cornerIndicatorsFrame = frame
	frame:Hide()
	frame:SetAllPoints(overlay)
	frame.cornerIndicators = {}
	local key
	for _, key in ipairs(self.INDICATOR_KEYS) do
		local indicator = self:CreateIndicator(frame, key)
		frame.cornerIndicators[key] = indicator
		self:UpdateIndicator(indicator, 1, 1, 1, 1)
	end

	frame:RegisterEvent("UNIT_AURA")
	frame:RegisterEvent("UNIT_FLAGS")
	frame:RegisterEvent("UNIT_CONNECTION")
	frame:SetScript("OnShow", Frame_OnShow)
	frame:SetScript("OnHide", Frame_OnHide)
end

-- Class categories: 1 physical, 2 magical
local CLASS_CATEGORIES = {
	WARRIOR = 1,
	ROGUE = 1,
	DEATHKNIGHT = 1,
	HUNTER = 1,
	PRIEST = 2,
	MAGE = 2,
	WARLOCK = 2,
}

function module:OnUnitChange(overlay, unit, realUnit, inVehicle, guid)
	local frame = overlay.cornerIndicatorsFrame
	frame.unit = unit
	if not unit or not realUnit then
		frame:Hide()
		return
	end

	local class = realUnit and select(2, UnitClass(realUnit))
	frame.class = class
	frame.inVehicle = inVehicle or (not UnitIsPlayer(unit) and UnitPlayerControlled(unit))
	frame.category = CLASS_CATEGORIES[class]
	frame.friendly = UnitIsFriend(unit, "player")

	if class == "DRUID" then
		frame:RegisterEvent("UNIT_DISPLAYPOWER") -- Druid class category varies, physical in bear/cat forms, magical otherwise
	else
		frame:UnregisterEvent("UNIT_DISPLAYPOWER")
	end

	frame:Show()
	Frame_UpdateAura(frame)
end

function module:OnRangeChange(overlay, inRange)
	local frame = overlay.cornerIndicatorsFrame
	frame.inRange = inRange
	Frame_UpdateAura(frame)
end

function module:OnIconsChange(overlay, hasRaidIcon, hasRoleIcon)
	UpdateTopLeftPosition(overlay, hasRaidIcon, hasRoleIcon)
end

function module:OnRestoreDefaults()
	CompactRaid:CopyTable(self.talentdb, self.defaultsDB)
	self:UpdateAllIndicators(nil, 1, 1)
	self.optionPage.tabFrame:OnTabSelected(self.optionPage.tabFrame:GetSelection())
end