﻿------------------------------------------------------------
-- Core.lua
--
-- Abin
-- 2010/10/16
------------------------------------------------------------

local DEBUG = nil -- Debug mode 1/nil

local type = type
local pairs = pairs
local CreateFrame = CreateFrame
local GetLocale = GetLocale
local format = format
local wipe = wipe
local StaticPopup_Show = StaticPopup_Show
local tostring = tostring
local tinsert = tinsert
local tremove = tremove
local ipairs = ipairs
local InCombatLockdown = InCombatLockdown
local pcall = pcall
local DEFAULT_CHAT_FRAME = DEFAULT_CHAT_FRAME
local UNKNOWN = UNKNOWN

local addonName, addon = ... -- I won't use the first arg of ... as "CompactRaid", just in case I change it someday, who knows.
_G["CompactRaid"] = addon
addon.version = GetAddOnMetadata(addonName, "Version") or "1.0"
addon.numericVersion = tonumber(addon.version) or 1.0
addon.guid = "{8A6673B0-DEA9-40DA-A955-FB60FBD0ABAC}"
addon.title = "CompactRaid|TInterface\\OptionsFrame\\UI-OptionsFrame-NewFeatureIcon:0:0:0:-1|t" -- Playing a little cool stuff doesn't hurt...
addon.name = "CompactRaid"
addon.db = {}
addon.OBSOLETE_MODULES = { ["BarTextures"] = 1, ["RaidDebuff_Records"] = 1 } -- Obsolete modules, reject them from registering

local L = addon.L
local moduleLocales = {}
local modules = {} -- Stores all registered modules, accessible from no where else

local function Print(module, msg, r, g, b)
	if msg then
		local header = "CompactRaid"
		if type(module) == "table" then
			header = header.."-"..(module.title or module.name or UNKNOWN)
		end
		DEFAULT_CHAT_FRAME:AddMessage("|cffffff78"..header..":|r "..tostring(msg), r or 0.5, g or 0.75, b or 1) -- All modules use uniformed message color, please!
	end
end

function addon:Print(msg, r, g, b)
	Print(nil, msg, r, g, b)
end

function addon:CheckVersion(version, errorTitle)
	if type(version) == "string" then
		version = tonumber(version)
	end

	if type(version) ~= "number" or version <= addon.numericVersion then
		return 1 -- Passed
	end

	if errorTitle then
		addon:Print(format(L["version error"], tostring(errorTitle), tostring(version), addon.version), 1, 0, 0)
	end
end

function addon:GetModule(name)
	return modules[name]
end

function addon:GetOverlay(frame)
	return addon.buttonManager.GetOverlay(frame)
end

function addon:GetOverlayByGUID(guid)
	return addon.buttonManager.GetOverlayByGUID(guid)
end

function addon:RegisterLocale(name, locale, data)
	if type(name) == "string" and type(data) == "table" and (GetLocale() == locale or not moduleLocales[name]) then
		moduleLocales[name] = data
	end
end

function addon:GetLocale(name)
	return moduleLocales[name]
end

function addon:pcall(func, ...)
	if DEBUG then
		-- debug mode: captures errors normally
		func(...)
		return 1
	else
		-- release mode: prints errors on ChatFrame1 and let go
		local valid, err = pcall(func, ...)
		if not valid then
			self:Print(err, 1, 0, 0)
		end
		return valid, err
	end
end

-- Enumerate all created overlay frames, your callback function returning non-nil value will abort the process
function addon:EnumFrames(...)
	----------------------------------------------------------------
	-- The way how this function works depends on arguments list you pass:

	-- Arguments			-- Results (frame is the overlay frame that is passed to your callback function)
	----------------------------------------------------------------
	-- func, ...			-- func(frame, ...)		-- func is a valid function
	-- object, "method", ...	-- object:method(frame, ...)	-- object is a table, "method" is a member function name of object
	-- nil, "method", ...		-- frame:method(...)		-- first argument is nil, "method" is a member function name of the frame, such as "SetScale", "Show", "Hide", etc
	----------------------------------------------------------------
	return addon.buttonManager.EnumFrames(...)
end

local function Module_RegisterEvent(module, event, method)
	if type(method) ~= "function" and type(method) ~= "string" then
		method = nil
	end

	module.__eventFrame.events[event] = method
	if not module.__eventFrame:IsEventRegistered(event) then
		module.__eventFrame:RegisterEvent(event)
	end
end

local function Module_UnregisterEvent(module, event)
	module.__eventFrame.events[event] = nil
	if module.__eventFrame:IsEventRegistered(event) then
		module.__eventFrame:UnregisterEvent(event)
	end
end

local function Module_IsEventRegistered(module, event)
	return module.__eventFrame:IsEventRegistered(event)
end

local function Module_RegisterAllEvents(module)
	return module.__eventFrame:RegisterAllEvents()
end

local function Module_UnregisterAllEvents(module)
	return module.__eventFrame:UnregisterAllEvents()
end

local function EventFrame_OnEvent(self, event, ...)
	local parent = self.parent
	if type(parent.OnEvent) == "function" then
		parent:OnEvent(event, ...)
	else
		local func = self.events[event]
		if not func then
			func = parent[event]
		elseif type(func) ~= "function" then -- string, number, etc
			func = parent[func]
		end

		if type(func) == "function" then
			func(parent, ...)
		end
	end
end

-- Register a new module, it must have an unique name among all modules, and this function must be called before "VARIABLES_LOADED"!
function addon:RegisterModule(name, title, version, desc, dualTalents, parent)
	if type(name) ~= "string" or modules[name] or self.OBSOLETE_MODULES[name] then
		return
	end

	local module = {}
	modules[name] = module
	module.name = name
	module.title = title
	module.version = version
	module.desc = desc
	module.dualTalentSupport = dualTalents
	module.Print = Print
	module.pcall = addon.pcall
	module.RequestRestoreDefaults = addon.RestoreModuleDefaults
	module.CombatSafeCall = addon.CombatSafeCall
	module.CancelPendingCombatSafeCall = addon.CancelPendingCombatSafeCall

	if title then
		if type(parent) == "table" then
			parent = parent.title
		end

		module.parent = type(parent) == "string" and parent or addon.title
		module.optionPage = self.optionTemplates:CreateOptionFrame(nil, title, version, desc, dualTalents, module.parent)
		module.optionPage.moduleName = name
	end

	local frame = CreateFrame("Frame")
	module.__eventFrame = frame
	frame.parent = module
	frame.events = {}
	frame:SetScript("OnEvent", EventFrame_OnEvent)
	module.RegisterEvent = Module_RegisterEvent
	module.UnregisterEvent = Module_UnregisterEvent
	module.IsEventRegistered = Module_IsEventRegistered
	module.RegisterAllEvents = Module_RegisterAllEvents
	module.UnregisterAllEvents = Module_UnregisterAllEvents

	return module
end

function addon:EnumModules(func, ...)
	if type(func) == "function" then
		local name, module
		for name, module in pairs(modules) do
			func(name, module, ...)
		end
	end
end

function addon:CallModuleFunc(module, func, ...)
	func = module and module[func]
	if type(func) == "function" then
		-- Why pcall? One module's error won't halt the entire process. I do need a way to let the module know
		-- it's encountering errors, though, so just print out the error message. You got a better idea?
		module:pcall(func, module, ...)
	end
end

function addon:CallAllModulesFunc(func, ...)
	local module
	for _, module in pairs(modules) do
		addon:CallModuleFunc(module, func, ...)
	end
end

--[[
function addon:Debug()
	local count = self:EnumFrames(function(frame) print(frame:GetName()) end)
	print("Total frames: "..count)
end
--]]

------------------------------------------------------------
-- For modules' "Restore Defaults" features, new feature added in 0.2 beta
------------------------------------------------------------

local popupData = {
	button1 = OKAY,
	button2 = CANCEL,
	timeout = 0,
	exclusive = 1,
	whileDead = 1,
	hideOnEscape = 1,
	OnAccept = function(self, module, arg1)
		if type(module.OnRestoreDefaults) == "function" then
			if module:pcall(module.OnRestoreDefaults, module, arg1) then
				module:Print(text or L["defaults restored"])
			end
		end
	end,
}

StaticPopupDialogs[addon.guid] = popupData -- If you ever wondered why there's a guid for CompactRaid, here you go.

function addon:RestoreModuleDefaults(module, text, arg1)
	popupData.text = text or format(L["restore defaults text"], module.title or module.name or UNKNOWN)
	local dialog = StaticPopup_Show(addon.guid)
	if dialog then
		dialog.data = module
		dialog.data2 = arg1
		return dialog
	end
end

-- Clone all table contents from t2 to t1
function addon:CopyTable(t1, t2)
	if t1 == t2 or type(t1) ~= "table" or type(t2) ~= "table" then
		return
	end

	wipe(t1)
	local k, v
	for k, v in pairs(t2) do
		if type(v) == "table" then
			t1[k] = {}
			addon:CopyTable(t1[k], v)
		else
			t1[k] = v
		end
	end
	return t1
end

------------------------------------------------------------
-- For combat-safe calls, new feature added in 1.3
------------------------------------------------------------

local pendingCalls = {}

local function InvokeGlobalCall(func, ...)
	if type(func) == "string" then
		addon:pcall(_G[func], ...)
	else
		addon:pcall(func, ...)
	end
end

local function InvokeMethodCall(object, method, ...)
	addon:pcall(object[method], object, ...)
end

-- Issue a combat-safe call, it calls directly if not in combat and returns nothing,
-- or pending the call and returns the pending handle, the call will be invoked when
-- combat ends. The returned handle is only used to cancel the pending call.

-- handle = addon:CombatSafeCall("func", ...) -- Using a global function name
-- handle = addon:CombatSafeCall(func, ...) -- Using a function
-- handle = addon:CombatSafeCall(object, "method", ...) -- Using an object and its member method name

function addon:CombatSafeCall(object, method, ...)
	if InCombatLockdown() then
		local data = { object = object }
		data.isMethod = type(object) == "table"
		local count
		if data.isMethod then
			data.method = method
			count = select("#", ...)
		else
			count = select("#", method, ...)
		end

		if count > 0 then
			data.hasArgs = count
			local i
			for i = 1, count do
				if data.isMethod then
					data["arg"..i] = select(i, ...)
				else
					data["arg"..i] = select(i, method, ...)
				end
			end
		end

		tinsert(pendingCalls, data)
		return data
	else
		if type(object) == "table" then
			InvokeMethodCall(object, method, ...)
		else
			InvokeGlobalCall(object, method, ...)
		end
	end
end

-- Cancels a pending call
function addon:CancelPendingCombatSafeCall(handle)
	if InCombatLockdown() and type(data) == "table" then
		local id, data
		for id, data in ipairs(pendingCalls) do
			if data == handle then
				tremove(pendingCalls, id)
				return 1
			end
		end
	end
end

local frame = CreateFrame("Frame")
frame:RegisterEvent("PLAYER_REGEN_ENABLED")
frame:SetScript("OnEvent", function()
	local i, data
	for i = 1, #pendingCalls do -- It is important to keep the calling orders!
		local data = pendingCalls[i]
		if data.isMethod then
			if data.hasArgs then
				InvokeMethodCall(data.object, data.method, data.arg1, data.arg2, data.arg3, data.arg4, data.arg5, data.arg6, data.arg7, data.arg8, data.arg9, data.arg10, data.arg11, data.arg12, data.arg13, data.arg14, data.arg15, data.arg16, data.arg17, data.arg18, data.arg19, data.arg20)
			else
				InvokeMethodCall(data.object, data.method)
			end
		else
			if data.hasArgs then
				InvokeGlobalCall(data.object, data.arg1, data.arg2, data.arg3, data.arg4, data.arg5, data.arg6, data.arg7, data.arg8, data.arg9, data.arg10, data.arg11, data.arg12, data.arg13, data.arg14, data.arg15, data.arg16, data.arg17, data.arg18, data.arg19, data.arg20)
			else
				InvokeGlobalCall(data.object)
			end
		end
	end
	wipe(pendingCalls)
end)