------------------------------------------------------------
-- LayoutManager.lua
--
-- Abin
-- 2010/10/16
------------------------------------------------------------

local type = type
local pairs = pairs
local ipairs = ipairs
local tinsert = tinsert
local GetNumRaidMembers = GetNumRaidMembers
local GetNumPartyMembers = GetNumPartyMembers
local tinsert = tinsert

local _, addon = ...
local layout = {}
addon.layoutManager = layout

layout.partyAdjustFrames = { [CompactPartyFrameMember1] = 1, [CompactPartyFrameMember2] = 2, [CompactPartyFrameMember3] = 3, [CompactPartyFrameMember4] = 4, [CompactPartyFrameMember5] = 5 }
layout.containers = {}

local CONTAINER_EDGE = 12

function layout:UpdateContainers(method, ...)
	local frame
	for _, frame in ipairs(layout.containers) do
		local border = frame.border
		local func = border[method]
		if type(func) == "function" then
			func(border, ...)
		end
	end
end

local function Container_OnDragStart(self)
	if not addon.db.disableDragging then
		local parent = self:GetParent()
		if type(parent.OnDragStart) == "function" then
			parent:OnDragStart()
		end

		if parent.frameToDrag then
			parent.frameToDrag:StartMoving()
		end
	end
end

local function Container_OnDragStop(self)
	if not addon.db.disableDragging then
		local parent = self:GetParent()
		if parent.frameToDrag then
			parent.frameToDrag:StopMovingOrSizing()
		end

		if type(parent.OnDragStop) == "function" then
			parent:OnDragStop()
		end
	end
end

local function Container_UpdateInsets(self)
	local x, y = addon.db.containerHInset or CONTAINER_EDGE, addon.db.containerVInset or CONTAINER_EDGE
	self:ClearAllPoints()
	self:SetPoint("TOPLEFT", -x, y)
	self:SetPoint("BOTTOMRIGHT", x, -y)
end

local function Container_UpdateVisible(self)
	if addon.db.hideContainer then
		self:Hide()
	else
		self:Show()
	end
end

local function Container_UpdateAlpha(self)
	local alpha, borderAlpha = addon.db.containerAlpha or 100, addon.db.containerBorderAlpha or 100
	self:SetBackdropColor(1, 1, 1, alpha / 100)
	self:SetBackdropBorderColor(1, 1, 1, borderAlpha / 100)
end

local function ContainerBorder_OnShow(self)
	self:SetFrameStrata(self.strata or "BACKGROUND")
end

-- Create a layout container frame
function layout:CreateLayoutContainer(parent, enableMouse, frameToDrag, strata)
	parent:SetFrameStrata("LOW")
	local frame = CreateFrame("Frame", parent:GetName().."CompactRaidLayoutFrame", parent)
	local border = CreateFrame("Frame", frame:GetName().."Border", frame)

	border:SetBackdrop({ bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16, insets = {left = 5, right = 5, top = 5, bottom = 5 }})
	border.strata = strata
	border:SetFrameStrata(strata or "BACKGROUND")
	border:SetPoint("TOPLEFT", -CONTAINER_EDGE, CONTAINER_EDGE)
	border:SetPoint("BOTTOMRIGHT", CONTAINER_EDGE, -CONTAINER_EDGE)
	border:SetScript("OnShow", ContainerBorder_OnShow)

	frame.border = border
	tinsert(layout.containers, frame)

	if enableMouse then
		frame:SetClampedToScreen(true)
		border:EnableMouse(true)
		border:RegisterForDrag("LeftButton")
		border:SetScript("OnDragStart", Container_OnDragStart)
		border:SetScript("OnDragStop", Container_OnDragStop)

		if frameToDrag then
			frame.frameToDrag = frameToDrag
			frameToDrag:SetMovable(true)
		end
	end

	border.UpdateInsets = Container_UpdateInsets
	border.UpdateVisible = Container_UpdateVisible
	border.UpdateAlpha = Container_UpdateAlpha

	return frame
end

local function GroupDisableTitle(self)
	local fs = self.title:CreateFontString(self.title:GetName().."Text", "ARTWORK", "GameFontNormalSmall")
	fs:SetFont(STANDARD_TEXT_FONT, 10)
	self.title:SetFontString(fs)
	self.title:EnableMouse(false)
	self.title:SetScript("OnDragStart", nil)
	self.title:SetScript("OnDragStop", nil)
end

local partyPanel = CreateFrame("Frame", CompactPartyFrame:GetName().."PanelFrame", CompactPartyFrame)
partyPanel:SetPoint("TOPLEFT", 0, -14)
partyPanel:SetSize(72, 36)

RegisterStateDriver(CompactPartyFrame, "visibility", "[group:raid] hide; show")

function layout:UpdatePartyContainerSize()
	local height = 0
	local frame, id
	for frame, id in pairs(self.partyAdjustFrames) do
		if frame:IsVisible() then
			height = height + frame:GetHeight()
		end
	end

	partyPanel:SetSize(CompactPartyFrameMember1:GetWidth(), height)
	--CompactPartyFrame:SetClampRectInsets(0, 0, 0, height * (5 - maxId))
	return height
end

function layout:CheckFrameStatus(frame)
	if self.partyAdjustFrames[frame] then
		self:UpdatePartyContainerSize()
	end
end

-- Party frame container
local partyContainer = layout:CreateLayoutContainer(CompactPartyFrame, 1, CompactPartyFrame)
partyContainer:SetAllPoints(partyPanel)
GroupDisableTitle(CompactPartyFrame)
CompactPartyFrame:SetUserPlaced(true) -- Hah hah hah

-- Raid frame container
local raidContainer = layout:CreateLayoutContainer(CompactRaidFrameContainer, 1)
raidContainer:Hide()
raidContainer:SetPoint("TOPLEFT")

raidContainer:RegisterEvent("PLAYER_ENTERING_WORLD")
raidContainer:RegisterEvent("RAID_ROSTER_UPDATE")
raidContainer:SetScript("OnEvent", function(self)
	if GetNumRaidMembers() > 0 then
		self:Show()
	else
		self:Hide()
	end
end)

function raidContainer:OnDragStart(self)
	CompactRaidFrameManager_ResizeFrame_OnDragStart(CompactRaidFrameManager)
end

function raidContainer:OnDragStop(self)
	CompactRaidFrameManager.containerResizeFrame:StopMovingOrSizing()
	CompactRaidFrameManager_ResizeFrame_SavePosition(CompactRaidFrameManager)

end

-- Raid frame container size is dynamic
local function UpdateRaidContainerSize(self)
	if self == CompactRaidFrameContainer then
		local usedX, usedY = FlowContainer_GetUsedBounds(self)
		if usedX and usedY and usedX > 0 and usedY > 0 then
			raidContainer:SetSize(usedX, usedY)
			local resizer = CompactRaidFrameManager.containerResizeFrame
			resizer:SetClampRectInsets(0, usedX - resizer:GetWidth(), 0, resizer:GetHeight() - usedY)
		end
	end
end

hooksecurefunc("CompactRaidFrameContainer_UpdateBorder", UpdateRaidContainerSize)
hooksecurefunc("CompactRaidFrameManager_ResizeFrame_OnResizeStop", UpdateRaidContainerSize)

hooksecurefunc("CompactRaidGroup_OnLoad", function(self)
	if type(self) == "table" and type(self.GetName) == "function" then
		local name = self:GetName()
		if type(name) == "string" and strfind(name, "^CompactRaidGroup(%d+)$") then
			GroupDisableTitle(self)
		end
	end
end)

-- make CompactRaidFrameManager frame a little better looking...
local managerContainer = layout:CreateLayoutContainer(CompactRaidFrameManager, nil, nil, "MEDIUM")
managerContainer:SetAllPoints(CompactRaidFrameManager)
managerContainer.border:SetFrameLevel(1)
managerContainer.border:ClearAllPoints()
managerContainer.border:SetPoint("TOPLEFT")
managerContainer.border:SetPoint("BOTTOMRIGHT", -8, 0)
managerContainer.border.UpdateInsets = nil -- Border insets of this container is not changable!

CompactRaidFrameManager:SetBackdrop(nil)
CompactRaidFrameManager:SetFrameStrata("HIGH") -- So it covers all unit buttons when it expands
CompactRaidFrameManagerBorderTopLeft:Hide()
CompactRaidFrameManagerBorderTopRight:Hide()
CompactRaidFrameManagerBorderBottomLeft:Hide()
CompactRaidFrameManagerBorderBottomRight:Hide()
CompactRaidFrameManagerBorderTop:Hide()
CompactRaidFrameManagerBorderBottom:Hide()
CompactRaidFrameManagerBorderRight:Hide()
CompactRaidFrameManagerBg:Hide()

CompactRaidFrameManager.toggleButton:ClearAllPoints()
CompactRaidFrameManager.toggleButton:SetPoint("RIGHT", -12, 0)

CompactRaidFrameManagerDisplayFrame:EnableMouse(true)
CompactRaidFrameManagerDisplayFrame:ClearAllPoints()
CompactRaidFrameManagerDisplayFrame:SetPoint("TOPLEFT")
CompactRaidFrameManagerDisplayFrame:SetPoint("BOTTOMRIGHT", -11, 0)

CompactRaidFrameManagerDisplayFrameHeaderBackground:ClearAllPoints()
CompactRaidFrameManagerDisplayFrameHeaderBackground:SetPoint("TOPLEFT", 7, -6)
CompactRaidFrameManagerDisplayFrameHeaderBackground:SetPoint("TOPRIGHT", -3, -6)
CompactRaidFrameManagerDisplayFrameHeaderBackground:SetTexture(0, 0, 0, 0.6)

CompactRaidFrameManagerDisplayFrameHeaderDelineator:ClearAllPoints()
CompactRaidFrameManagerDisplayFrameHeaderDelineator:SetPoint("TOPLEFT", 0, -21)
CompactRaidFrameManagerDisplayFrameHeaderDelineator:SetPoint("TOPRIGHT", -1, -21)

CompactRaidFrameManagerDisplayFrameFooterDelineator:ClearAllPoints()
CompactRaidFrameManagerDisplayFrameFooterDelineator:SetPoint("TOPLEFT", 0, -102)
CompactRaidFrameManagerDisplayFrameFooterDelineator:SetPoint("TOPRIGHT", -1, -102)