------------------------------------------------------------
-- OptionFrame.lua
--
-- Abin
-- 2010/10/16
------------------------------------------------------------

local type = type
local SetCVar = SetCVar
local CompactRaidFrameContainer = CompactRaidFrameContainer
local InCombatLockdown = InCombatLockdown
local pairs = pairs
local GameTooltip = GameTooltip
local format = format
local floor = floor
local RegisterStateDriver = RegisterStateDriver
local UnregisterStateDriver = UnregisterStateDriver
local _G = _G
local TALENT_SPEC_PRIMARY = TALENT_SPEC_PRIMARY
local TALENT_SPEC_SECONDARY = TALENT_SPEC_SECONDARY

local _, addon = ...
local L = addon.L
local manager = addon.buttonManager
local layout = addon.layoutManager
local templates = addon.optionTemplates

local frame = templates:CreateOptionFrame("CompactRaidOptionPage", addon.title, addon.version, L["desc"])
addon.optionPage = frame
frame.moduleName = "CompactRaid"
frame.sampleX, frame.sampleY = 314, -75

-- For down-ward compatibility in case the user did not update their modules to new versions.
frame.SetEditBoxAutoPrompt = templates.SetEditBoxAutoPrompt
frame.CreateScaleOffsetGroup = templates.CreateScaleOffsetGroup

function frame:OnSampleShow()
	addon.sampleOverlay:Hide()
end

function frame:OnSampleHide()
	addon.sampleOverlay:Show()
end

local scrollFrame = templates:CreateScrollFrame(frame:GetName().."ScrollFrame", frame)
frame.scrollFrame = scrollFrame
scrollFrame:SetPoint("TOPLEFT", 16, -70)
scrollFrame:SetPoint("TOPRIGHT", -28, -70)
scrollFrame:SetHeight(310)

local page = UICreateInterfaceOptionPage(frame:GetName().."ScrollChild", "", nil, nil, frame)
frame.scrollChild = page
page:Show()
page:SetSize(10, 10)
scrollFrame:SetScrollChild(page)

local function UpdateSingleMode()
	if addon.db.single then
		RegisterStateDriver(CompactPartyFrame, "visibility", "[group:raid] hide; show")
	else
		UnregisterStateDriver(CompactPartyFrame, "visibility")
		CompactPartyFrame_UpdateShown(CompactPartyFrame)
	end
end

------------------------------------------------------------
-- General option items
------------------------------------------------------------

local anchor
local group = page:CreateMultiSelectionGroup(L["general options"])
group:SetPoint("TOPLEFT")
group:AddButton(L["show when single-playing"], "single", 1)
group:AddButton(L["invert bar color"], "invertColor")
group:AddButton(L["display raid icons"], "showRaidIcon")

local lastButtonOfGeneralGroup = group[-1]

function group:OnCheckInit(value)
	return addon.db[value]
end

function group:OnCheckChanged(value, checked)
	addon.db[value] = checked

	if value == "single" then
		UpdateSingleMode()

	elseif value == "invertColor" then
		manager.CallAllButtonFunc(manager.UpdateHealthColor)
		manager.CallAllButtonFunc(manager.UpdatePowerColor)
		manager.UpdateHealthColor(addon.sampleButton)
		manager.UpdatePowerColor(addon.sampleButton)

	elseif value == "showRaidIcon" then
		manager.CallAllButtonFunc(manager.OnUpdateRaidIcon)
		addon.sampleButton:UpdateRaidIcon()

	end
end

local function UpdateScale()
	local scale = addon.db.scale / 100
	CompactRaidFrameContainer:SetScale(scale)
	CompactRaidFrameManagerContainerResizeFrame:SetScale(scale)
	CompactPartyFrame:SetScale(scale)
end

local scaleSlider = page:CreateSmallSlider(L["frame scale"], 25, 200, 5, "%d%%", 1)
scaleSlider:SetPoint("TOPLEFT", lastButtonOfGeneralGroup, "BOTTOMLEFT", 8, -74)

function scaleSlider:OnSliderInit()
	return addon.db.scale
end

function scaleSlider:OnSliderChanged(value)
	addon.db.scale = value
	addon.sampleButtonPanel:SetScale(value / 100)
	addon:CombatSafeCall(UpdateScale)
end

local powerBarSlider = page:CreateSmallSlider(L["mana height"], 0, 10)
powerBarSlider:SetPoint("LEFT", scaleSlider, "RIGHT", 20, 0)

function powerBarSlider:OnSliderInit()
	return addon.db.powerBarHeight
end

function powerBarSlider:OnSliderChanged(value)
	addon.db.powerBarHeight = value
	manager.CallAllButtonFunc(manager.UpdatePowerBarHeight)
	manager.UpdatePowerBarHeight(addon.sampleButton)
end

------------------------------------------------------------
-- Aura options
------------------------------------------------------------

local function BuffScaleOffsetCallback(overlay, key)
	manager.SetBuffFrameScalePoint(overlay:GetParent(), key)
end

local function CreateBuffScaleOffsetFrame(key, group)
	local frame = templates:CreateScaleOffsetGroup(page, nil, HIGHLIGHT_FONT_COLOR)
	frame.buffKey = key
	frame:SetPoint("TOPLEFT", group[-1].text, "BOTTOMLEFT", 0, -4)

	frame.editboxes.scale.defaultColor = HIGHLIGHT_FONT_COLOR
	frame.editboxes.xoffset.defaultColor = HIGHLIGHT_FONT_COLOR
	frame.editboxes.yoffset.defaultColor = HIGHLIGHT_FONT_COLOR

	frame.OnScaleApply = function(self, scale)
		addon.db[self.buffKey.."Scale"] = scale
		addon:EnumFrames(BuffScaleOffsetCallback, self.buffKey)
		BuffScaleOffsetCallback(addon.sampleOverlay, self.buffKey)
	end

	frame.OnOffsetApply = function(self, x, y)
		addon.db[self.buffKey.."XOffset"], addon.db[self.buffKey.."YOffset"] = x, y
		addon:EnumFrames(BuffScaleOffsetCallback, self.buffKey)
		BuffScaleOffsetCallback(addon.sampleOverlay, self.buffKey)
	end

	frame.OnScaleCancel = function(self)
		return addon.db[self.buffKey.."Scale"]
	end

	frame.OnOffsetCancel = function(self)
		return addon.db[self.buffKey.."XOffset"], addon.db[self.buffKey.."YOffset"]
	end

	frame:HookScript("OnShow", function(self)
		self:SetValue("scale", addon.db[self.buffKey.."Scale"])
		self:SetValue("offset", addon.db[self.buffKey.."XOffset"], addon.db[self.buffKey.."YOffset"])
	end)

	return frame
end

group = page:CreateMultiSelectionGroup(L["aura options"])
group:SetPoint("TOPLEFT", lastButtonOfGeneralGroup, "BOTTOMLEFT", 0, -110)

group:AddButton(L["hide buffs"], "hideBuffs")
local buffScaleOffset = CreateBuffScaleOffsetFrame("buff", group)

group:AddButton(L["hide debuffs"], "hideDebuffs")
group[-1]:ClearAllPoints()
group[-1]:SetPoint("TOPLEFT", group[1], "BOTTOMLEFT", 0, -26)
local debuffScaleOffset = CreateBuffScaleOffsetFrame("debuff", group)

group:AddButton(L["hide dispels"], "hideDispels")
group[-1]:ClearAllPoints()
group[-1]:SetPoint("TOPLEFT", group[2], "BOTTOMLEFT", 0, -26)
local dispelScaleOffset = CreateBuffScaleOffsetFrame("dispelDebuff", group)

local lastButtonOfAuraGroup = group[-1]

function group:OnCheckInit(value)
	return addon.db[value]
end

function group:OnCheckChanged(value, checked)
	addon.db[value] = checked

	if value == "hideBuffs" then
		addon.sampleButton.buff:UpdateVisible(checked)
		manager.CallAllButtonFunc(function(self)
			if self.displayedUnit then
				CompactUnitFrame_UpdateAuras(self)
			end
		end)

	elseif value == "hideDebuffs" then
		addon.sampleButton.debuff:UpdateVisible(checked)
		manager.CallAllButtonFunc(function(self)
			if self.displayedUnit then
				CompactUnitFrame_UpdateAuras(self)
			end
		end)

	elseif value == "hideDispels" then
		addon.sampleButton.dispel:UpdateVisible(checked)
		manager.CallAllButtonFunc(function(self)
			if self.displayedUnit then
				CompactUnitFrame_UpdateAuras(self)
			end
		end)
	end
end

------------------------------------------------------------
-- Tooltip options
------------------------------------------------------------

group = page:CreateSingleSelectionGroup(L["Display tooltips"], 1)
group:SetPoint("TOPLEFT", lastButtonOfAuraGroup, "BOTTOMLEFT", 0, -64)
group:AddButton(ALWAYS, 2)
group:AddButton(L["out of combat"], 1)
group:AddButton(NEVER, 0)

function group:OnCheckInit(value)
	return addon.db.showtooltip == value
end

function group:OnSelectionChanged(value)
	addon.db.showtooltip = value
end

------------------------------------------------------------
-- Health bar color options
------------------------------------------------------------

-- just to simplify the code...
local function CreateColorSelectionGroup(text, dbKey, dbColorKey, buttonFunc, hasVehicle, vehicleKey, vehicleColorKey)
	local group = templates:CreateClassCustomColorGroup(page, text, hasVehicle)
	group.dbKey, group.dbColorKey, group.buttonFunc, group.vehicleKey, group.vehicleColorKey = dbKey, dbColorKey, manager[buttonFunc], vehicleKey, vehicleColorKey

	group.OnColorRequest = function(self, vehicle)
		local key = vehicle and self.vehicleColorKey or self.dbColorKey
		return addon.db[key.."r"], addon.db[key.."g"], addon.db[key.."b"]
	end

	group.OnColorChange = function(self, r, g, b, vehicle)
		local key = vehicle and self.vehicleColorKey or self.dbColorKey
		addon.db[key.."r"], addon.db[key.."g"], addon.db[key.."b"] = r, g, b
	end

	group.OnOptionInit = function(self, value, vehicle)
		if vehicle then
			return addon.db[self.vehicleKey]
		else
			return addon.db[self.dbKey] == value
		end
	end

	group.OnOptionChange = function(self, value, vehicle)
		if vehicle then
			addon.db[self.vehicleKey] = value
		else
			addon.db[self.dbKey] = value
		end
	end

	group.OnApplyOption = function(self, vehicle)
		manager.CallAllButtonFunc(self.buttonFunc)
		self.buttonFunc(addon.sampleButton)
	end

	return group
end

anchor = group[1]
group = CreateColorSelectionGroup(L["health bar color"], "healthClass", "health", "UpdateHealthColor", 1, "useVehicleColor", "vehicle")
group:SetPoint("TOPLEFT", anchor, "BOTTOMLEFT", 0, -12)

------------------------------------------------------------
-- Name text options
------------------------------------------------------------

anchor = group[1]
group = CreateColorSelectionGroup(L["name text options"], "nameClass", "name", "UpdateNameColor")
group:SetPoint("TOPLEFT", anchor, "BOTTOMLEFT", 0, -12)
anchor = group[1]

local function CreateNameTextOptionSlider(text, minVal, maxVal, step, valueFormat, dbKey, buttonFunc)
	local slider = page:CreateSmallSlider(text, minVal, maxVal, step, valueFormat)
	slider.dbKey, slider.buttonFunc = dbKey, manager[buttonFunc]

	slider.OnSliderInit = function(self)
		return addon.db[self.dbKey] or 0
	end

	slider.OnSliderChanged = function(self, value)
		addon.db[self.dbKey] = value
		manager.CallAllButtonFunc(self.buttonFunc)
		self.buttonFunc(addon.sampleButton)
	end
	return slider
end

local nameLenSlider = CreateNameTextOptionSlider(L["text length"], 0, 100, 5, "%d%%", "nameWidthLimit", "OnNameTextSet")
nameLenSlider:SetPoint("TOPLEFT", anchor, "BOTTOMLEFT", 8, -26)

local nameHeightSlider = CreateNameTextOptionSlider(L["text height"], 2, 20, nil, nil, "nameHeight", "OnNameTextHeight")
nameHeightSlider:SetPoint("LEFT", nameLenSlider, "RIGHT", 20, 0)

local nameXOffSlider = CreateNameTextOptionSlider(L["x-offset"], -20, 20, nil, nil, "nameXOffset", "OnNameTextOffset")
nameXOffSlider:SetPoint("TOPLEFT", nameLenSlider, "BOTTOMLEFT", 0, -34)

local nameYOffSlider = CreateNameTextOptionSlider(L["y-offset"], -20, 20, nil, nil, "nameYOffset", "OnNameTextOffset")
nameYOffSlider:SetPoint("LEFT", nameXOffSlider, "RIGHT", 20, 0)

------------------------------------------------------------
-- Frame container options
------------------------------------------------------------

group = page:CreateMultiSelectionGroup(L["frame container options"], 1)
group:SetPoint("TOPLEFT", anchor, "BOTTOMLEFT", 0, -110)
group:AddButton(L["hide frame container"], "hideContainer")
group:AddButton(L["Disable mouse dragging"], "disableDragging")

function group:OnCheckInit(value)
	return addon.db[value]
end

function group:OnCheckChanged(value, checked)
	addon.db[value] = checked
	if value == "hideContainer" then
		layout:UpdateContainers("UpdateVisible")
	end
end

-- Background Alpha
local containerAlphaSlider = page:CreateSmallSlider(L["background alpha"], 0, 100, 5, "%d%%")
containerAlphaSlider:SetPoint("TOPLEFT", group[1], "BOTTOMLEFT", 8, -26)

function containerAlphaSlider:OnSliderInit()
	return addon.db.containerAlpha
end

function containerAlphaSlider:OnSliderChanged(value)
	addon.db.containerAlpha = value
	layout:UpdateContainers("UpdateAlpha")
end

-- Border alpha
local containerBorderAlphaSlider = page:CreateSmallSlider(L["border alpha"], 0, 100, 5, "%d%%")
containerBorderAlphaSlider:SetPoint("LEFT", containerAlphaSlider, "RIGHT", 20, 0)

function containerBorderAlphaSlider:OnSliderInit()
	return addon.db.containerBorderAlpha
end

function containerBorderAlphaSlider:OnSliderChanged(value)
	addon.db.containerBorderAlpha = value
	layout:UpdateContainers("UpdateAlpha")
end

-- Horizontal insets
local containerHInsetSlider = page:CreateSmallSlider(L["hori inset"], 0, 30)
containerHInsetSlider:SetPoint("TOPLEFT", containerAlphaSlider, "BOTTOMLEFT", 0, -34)

function containerHInsetSlider:OnSliderInit()
	return addon.db.containerHInset
end

function containerHInsetSlider:OnSliderChanged(value)
	addon.db.containerHInset = value
	layout:UpdateContainers("UpdateInsets")
end

-- Vertical insets
local containerVInsetSlider = page:CreateSmallSlider(L["vert inset"], 0, 30)
containerVInsetSlider:SetPoint("LEFT", containerHInsetSlider, "RIGHT", 20, 0)

function containerVInsetSlider:OnSliderInit()
	return addon.db.containerVInset
end

function containerVInsetSlider:OnSliderChanged(value)
	addon.db.containerVInset = value
	layout:UpdateContainers("UpdateInsets")
end

------------------------------------------------------------
-- Internal untility functions
------------------------------------------------------------

local function NormalizeNumber(number, low, high, default)
	if type(number) ~= "number" then
		return default
	end

	number = floor(number + 0.5)
	return number >= low and number <= high and number or default
end

local function NotifyTalentChange(name, module, talentGroup)
	if not module.dualTalentSupport then
		return
	end

	local talentName = talentGroup == 2 and TALENT_SPEC_SECONDARY or TALENT_SPEC_PRIMARY
	local page = module.optionPage
	if page and page.dualTalentText then
		page.dualTalentText:SetText("<"..talentName..">")
	end

	local firstTime
	local key = "talent"..talentGroup
	module.talentdb = module.chardb[key]
	if type(module.talentdb) ~= "table" then
		firstTime = 1
		module.talentdb = {}
		module.chardb[key] = module.talentdb

		local other = module.chardb[talentGroup == 1 and "talent2" or "talent1"]
		if type(other) == "table" then
			addon:CopyTable(module.talentdb, other)
		end
	end

	if type(module.OnTalentGroupChange) == "function" then
		if module:pcall(module.OnTalentGroupChange, module, talentGroup, module.talentdb, firstTime) then
			module:Print(text or format(L["talent group message"], talentName))
		end
	end
end

local function InitializeModule(name, module, db, chardb)
	local firstTime
	if type(db[name]) ~= "table" then
		firstTime = 1
		db[name] = {}
	end
	module.db = db[name]

	if type(chardb[name]) ~= "table" then
		firstTime = 1
		chardb[name] = {}
	end
	module.chardb = chardb[name]

	addon:CallModuleFunc(module, "OnInitialize", module.db, module.chardb, firstTime)
end

local function CleanupObsoletedModules(db)
	local name
	for name in pairs(addon.OBSOLETE_MODULES) do
		db[name] = nil
	end
end

------------------------------------------------------
-- A private event frame which keeps tracking players raty/raid
-- status and triggers modules "OnEnable/OnDisable" functions
------------------------------------------------------
local curEDStatus
local function CheckEDStatus()
	local oldStatus = curEDStatus
	if GetNumRaidMembers() > 0 then
		curEDStatus = "raid"
	elseif GetNumPartyMembers() > 0 then
		curEDStatus = "party"
	else
		curEDStatus = nil
	end

	if oldStatus ~= curEDStatus then

		if oldStatus then
			addon:CallAllModulesFunc("OnLeaveGroup", oldStatus)
		end

		if curEDStatus then
			addon:CallAllModulesFunc("OnJoinGroup", curEDStatus)
		end
	end
end

local EDFrame = CreateFrame("Frame")
EDFrame:SetScript("OnEvent", CheckEDStatus)

local function UpdateDBVersion(db)
	local dbVer = tonumber(db.version or 0) or 0
	db.version = addon.numericVersion
	return dbVer
end

frame:RegisterEvent("VARIABLES_LOADED")
frame:SetScript("OnEvent", function(self, event, ...)
	if event == "VARIABLES_LOADED" then

		addon:Print(format(L["load prompt"], addon.version or UNKNOWN))
		SetCVar("raidFramesDisplayIncomingHeals", "0") -- Just to save some CPU resource

		------------------------------------------------------------
		-- Assign slash command "/CompactRaid"
		------------------------------------------------------------

		SLASH_COMPACTRAID1 = "/compactraid"
		SLASH_COMPACTRAID2 = "/craid"
		SlashCmdList["COMPACTRAID"] = function() addon.optionPage:Open() end

		------------------------------------------------------------
		-- Prepare user data
		------------------------------------------------------------

		if type(CompactRaidDB) ~= "table" then
			CompactRaidDB = { useVehicleColor = 1, single = 1, showRaidIcon = 1, namer = 1, nameg = 1, nameb = 1 }
		end
		addon.db = CompactRaidDB

		-- Data version tracking
		local originalVersion = UpdateDBVersion(addon.db)

		if type(addon.db.samplePositions) ~= "table" then
			addon.db.samplePositions = {}
		end

		if type(addon.db.modules) ~= "table" then
			addon.db.modules = {}
		end

		-- Character private data
		if type(CompactRaidCharDB) ~= "table" then
			CompactRaidCharDB = {}
		end

		addon.chardb = CompactRaidCharDB

		if type(addon.chardb.modules) ~= "table" then
			addon.chardb.modules = {}
		end

		------------------------------------------------------------
		-- Cleanup data saved for obsoleted modules
		------------------------------------------------------------

		CleanupObsoletedModules(addon.db.modules)
		CleanupObsoletedModules(addon.chardb.modules)
		CleanupObsoletedModules(addon.db.samplePositions)

		------------------------------------------------------------
		-- Validate & apply general option data
		------------------------------------------------------------

		addon.db.scale = NormalizeNumber(addon.db.scale, 5, 300, 100)
		addon.db.powerBarHeight = NormalizeNumber(addon.db.powerBarHeight, 0, 10, 5)
		addon.db.containerAlpha = NormalizeNumber(addon.db.containerAlpha, 0, 100, 75)
		addon.db.containerBorderAlpha = NormalizeNumber(addon.db.containerBorderAlpha, 0, 100, 100)
		addon.db.showtooltip = NormalizeNumber(addon.db.showtooltip, 0, 2, 1)
		addon.db.nameWidthLimit = NormalizeNumber(addon.db.nameWidthLimit, 0, 100, 75)
		addon.db.nameHeight = NormalizeNumber(addon.db.nameHeight, 2, 20, 12)
		addon.db.healthClass = NormalizeNumber(addon.db.healthClass, 0, 1, 1)
		addon.db.nameClass = NormalizeNumber(addon.db.nameClass, 0, 1, 1)
		addon.db.containerHInset = NormalizeNumber(addon.db.containerHInset, 0, 30, 12)
		addon.db.containerVInset = NormalizeNumber(addon.db.containerVInset, 0, 30, 12)

		layout:UpdateContainers("UpdateVisible")
		layout:UpdateContainers("UpdateAlpha")
		layout:UpdateContainers("UpdateInsets")

		addon:CombatSafeCall(UpdateSingleMode)
		addon:CombatSafeCall(UpdateScale)

		------------------------------------------------------------
		-- Setup our sample button
		------------------------------------------------------------

		manager.OnNameTextSet(addon.sampleButton)
		manager.UpdatePowerBarHeight(addon.sampleButton)
		addon.sampleButtonPanel:SetScale(addon.db.scale / 100)
		addon.sampleButton:UpdateRaidIcon()
		manager.OnNameTextInit(addon.sampleButton)

		addon.sampleButton.buff:UpdateVisible(addon.db.hideBuffs)
		addon.sampleButton.debuff:UpdateVisible(addon.db.hideDebuffs)
		addon.sampleButton.dispel:UpdateVisible(addon.db.hideDispels)

		BuffScaleOffsetCallback(addon.sampleOverlay, "buff")
		BuffScaleOffsetCallback(addon.sampleOverlay, "debuff")
		BuffScaleOffsetCallback(addon.sampleOverlay, "dispelDebuff")

		------------------------------------------------------------
		-- Wake up our modules, call their "OnInitialize"
		------------------------------------------------------------

		addon:EnumModules(InitializeModule, addon.db.modules, addon.chardb.modules)
		addon:EnumModules(NotifyTalentChange, GetActiveTalentGroup() or 1)
		self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")

		------------------------------------------------------------
		-- Start triggering OnEnable/OnDisable events
		------------------------------------------------------------

		EDFrame:RegisterEvent("PARTY_MEMBERS_CHANGED")
		EDFrame:RegisterEvent("RAID_ROSTER_UPDATE")
		CheckEDStatus()

		------------------------------------------------------------
		-- Initialize addon and install necessary hooks
		------------------------------------------------------------

		manager.DoInitialize() -- Never call this function more than once per game session!

	elseif event == "ACTIVE_TALENT_GROUP_CHANGED" then
		addon:EnumModules(NotifyTalentChange, GetActiveTalentGroup() or 1)
	end
end)

------------------------------------------------------------
-- Make Blizzard Compactraid manager frame button opens our option page on right-click
------------------------------------------------------------

CompactRaidFrameManagerDisplayFrameOptionsButton:RegisterForClicks("LeftButtonUp", "RightButtonUp")
CompactRaidFrameManagerDisplayFrameOptionsButton:HookScript("OnClick", function(self, button)
	if button == "RightButton" then
		frame:Open()
	end
end)

CompactRaidFrameManagerDisplayFrameOptionsButton:HookScript("OnEnter", function(self)
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	GameTooltip:ClearLines()
	GameTooltip:AddLine("CompactRaid", 1, 1, 1)
	GameTooltip:AddLine(L["hooked tooltip"], nil, nil, nil, 1)
	GameTooltip:Show()
end)

CompactRaidFrameManagerDisplayFrameOptionsButton:HookScript("OnLeave", function(self)
	GameTooltip:Hide()
end)

------------------------------------------------------------
-- Add 2 quick browse buttons to Blizzard's "Raid Frames" and our own option pages
------------------------------------------------------------

local function CreateQuickBrowseButton(name, parent, key)
	local button = CreateFrame("Button", name, parent, "UIPanelButtonTemplate2")
	button.key = key
	button:SetWidth(140)
	button:SetText(key)
	button:SetScript("OnClick", function(self) InterfaceOptionsFrame_OpenToCategory(self.key) end)
	return button
end

local browseButton1 = CreateQuickBrowseButton("CompactRaidBrowseToButton1", InterfaceOptionsRaidFramePanel, addon.title)
browseButton1:SetPoint("LEFT", InterfaceOptionsRaidFramePanelResetPosition, "RIGHT", 90, 0)

local browseButton2 = CreateQuickBrowseButton("CompactRaidBrowseToButton2", frame, RAID_FRAMES_LABEL)
browseButton2:SetPoint("TOPLEFT", scrollFrame, "BOTTOMLEFT", 233, -11)

------------------------------------------------------------
-- Mandatory settings
------------------------------------------------------------

templates:DisableBlizzardOption(InterfaceOptionsRaidFramePanelDisplayBorder, "0") -- If you don't agree THIS one with me, you are not worth my efforts, seriously.
templates:DisableBlizzardOption(InterfaceOptionsRaidFramePanelDisplayClassColor, "0") -- Option over-ridden by Compactraid's "Health Bar Color" settings
templates:DisableBlizzardOption(InterfaceOptionsRaidFramePanelDisplayPowerBars, "1") -- Option over-ridden by Compactraid's "Power Bar height" settings
templates:DisableBlizzardOption(InterfaceOptionsRaidFramePanelDisplayIncomingHeals) -- Option over-ridden by Compactraid's internal framework

------------------------------------------------------------
-- Steal Blizzard's option frame controls
------------------------------------------------------------

page:HookScript("OnShow", function(self)
	local xSlider, ySlider, checkbox, data
	xSlider, data = templates:StealBlizzardSlider(InterfaceOptionsRaidFramePanelRaidFrameWidthSlider, self)
	data.text:SetText(L["unit width"])
	xSlider:SetPoint("TOPLEFT", lastButtonOfGeneralGroup, "BOTTOMLEFT", 8, -24)

	ySlider, data = templates:StealBlizzardSlider(InterfaceOptionsRaidFramePanelRaidFrameHeightSlider, self)
	data.text:SetText(L["unit height"])
	ySlider:SetPoint("LEFT", xSlider, "RIGHT", 20, 0)

	checkbox = templates:StealBlizzardOptionFrame(InterfaceOptionsRaidFramePanelDisplayOnlyDispellableDebuffs, self)
	checkbox:SetPoint("TOPLEFT", lastButtonOfAuraGroup, "BOTTOMLEFT", 0, -26)
end)

page:HookScript("OnHide", function(self)
	templates:ReturnBlizzardSlider(InterfaceOptionsRaidFramePanelRaidFrameHeightSlider)
	templates:ReturnBlizzardSlider(InterfaceOptionsRaidFramePanelRaidFrameWidthSlider)
	templates:ReturnBlizzardOptionFrame(InterfaceOptionsRaidFramePanelDisplayOnlyDispellableDebuffs)
end)